package com.yunfeiyun.agmp.common.web.system.service.impl;

import com.yunfeiyun.agmp.common.config.SystemConfig;
import com.yunfeiyun.agmp.common.constant.ErrorCode;
import com.yunfeiyun.agmp.common.constant.ResConstants;
import com.yunfeiyun.agmp.common.constant.UserConstants;
import com.yunfeiyun.agmp.common.exception.BizException;
import com.yunfeiyun.agmp.common.framework.web.service.ISysUserService;
import com.yunfeiyun.agmp.common.framework.web.service.PolicyService;
import com.yunfeiyun.agmp.common.web.system.domain.SysRes;
import com.yunfeiyun.agmp.common.web.system.mapper.PtsUserBindMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysRoleMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysUserMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysUserRoleMapper;
import com.yunfeiyun.agmp.common.web.system.service.ISysConfigService;
import com.yunfeiyun.agmp.common.core.domain.entity.PtsUserBind;
import com.yunfeiyun.agmp.common.core.domain.entity.SysRole;
import com.yunfeiyun.agmp.common.core.domain.entity.SysUser;
import com.yunfeiyun.agmp.common.core.domain.entity.SysUserRole;
import com.yunfeiyun.agmp.common.core.domain.model.IBaseUser;
import com.yunfeiyun.agmp.common.core.domain.model.IBaseUserBind;
import com.yunfeiyun.agmp.common.core.domain.model.IUserRole;
import com.yunfeiyun.agmp.common.framework.manager.ResManager;
import com.yunfeiyun.agmp.common.utils.ConfigUtils;
import com.yunfeiyun.agmp.common.utils.DateUtils;
import com.yunfeiyun.agmp.common.utils.SecurityUtils;
import com.yunfeiyun.agmp.common.utils.StringUtils;
import com.yunfeiyun.agmp.common.utils.bean.BeanValidators;
import com.yunfeiyun.agmp.common.utils.file.FileUploadUtils;
import com.yunfeiyun.agmp.common.utils.file.MimeTypeUtils;
import com.yunfeiyun.agmp.common.utils.spring.SpringUtils;
import com.yunfeiyun.agmp.common.utils.uuid.IdUtils;
import com.yunfeiyun.agmp.common.utils.uuid.Uuids;
import com.yunfeiyun.agmp.common.web.system.service.ISysResService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Validator;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户 业务层处理
 * 
 * @author 杨晓辉
 */
@Service("sysUserService")
public class SysUserServiceImpl implements ISysUserService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    protected Validator validator;

    @Resource
    private ISysResService sysResService;

    @Autowired
    private ResManager resManager;
    /**
     * 根据条件分页查询用户列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    //@DataScope(deptAlias = "d", userAlias = "u")  禁用数据权限，请知晓
    public List<IBaseUser> selectUserList(IBaseUser user)
    {
        return sysUserMapper.selectUserList(user);
    }

    @Override
    public List<IBaseUser> selectUserListOfAuth(IBaseUser sysUser) {
        return sysUserMapper.selectUserListOfAuth(sysUser);
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    //@DataScope(deptAlias = "d", userAlias = "u") 禁用数据权限，请知晓
    public List<IBaseUser> selectAllocatedList(IBaseUser user)
    {
        return sysUserMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    //@DataScope(deptAlias = "d", userAlias = "u") 禁用数据权限，请知晓
    public List<IBaseUser> selectUnallocatedList(IBaseUser user)
    {
        return sysUserMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     * 
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public IBaseUser selectUserByUserName(String userName)
    {
        return sysUserMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     * 
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public IBaseUser selectUserById(String userId)
    {
        return sysUserMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 查询用户所属岗位组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName)
    {
       return null;
    }

    /**
     * 校验用户名称是否唯一
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(IBaseUser user)
    {
        String userId = StringUtils.isNull(user.getUserId()) ? "-1" : user.getUserId();
        IBaseUser info = sysUserMapper.checkUserNameUnique(user.getUserLoginname());
        if (StringUtils.isNotNull(info) && !info.getUserId().equalsIgnoreCase(userId))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkPhoneUnique(IBaseUser user)
    {
        String userId = StringUtils.isNull(user.getUserId()) ? "-1" : user.getUserId();
        IBaseUser info = sysUserMapper.checkPhoneUnique(user.getUserMobile());
        if (StringUtils.isNotNull(info) && !info.getUserId().equalsIgnoreCase(userId))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkEmailUnique(IBaseUser user)
    {
        String userId = StringUtils.isNull(user.getUserId()) ? "-1" : user.getUserId();
        IBaseUser info = (SysUser) sysUserMapper.checkEmailUnique(user.getUserEmail());
        if (StringUtils.isNotNull(info) && !info.getUserId().equalsIgnoreCase( userId))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     * 
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(IBaseUser user)
    {
        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
        {
            throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),"不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     * 
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(String userId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysUser user = new SysUser("");
            user.setUserId(userId);
            List<IBaseUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
            if (StringUtils.isEmpty(users))
            {
                throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),"没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(IBaseUser user)
    {
        // 新增用户信息

        int rows = sysUserMapper.insertUser(user);
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    /**
     * 注册用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(IBaseUser user)
    {
        return sysUserMapper.insertUser(user) > 0;
    }

    /**
     * 修改保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(IBaseUser user)
    {
        String userId = user.getUserId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(user);
        // 新增用户与岗位管理
        insertUserPost(user);
        return sysUserMapper.updateUser(user);
    }

    /**
     * 用户授权角色
     * 
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(String userId, String[] roleIds)
    {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(IBaseUser user)
    {
        return sysUserMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(IBaseUser user)
    {
        return sysUserMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     * 
     * @param userId 用户Id
     * @param file 文件
     * @return 结果
     */
    @Override
    public SysRes updateUserAvatar(String userId, MultipartFile file)
    {
        SysRes sysRes = new SysRes();
        sysRes.setResId(sysRes.getId() );
        try {
            sysRes.setResBusId(userId);
            String avatar = FileUploadUtils.upload(SystemConfig.getAvatarPath(), file, MimeTypeUtils.IMAGE_EXTENSION, sysRes.getResId());
            String resOssUrl = resManager.uploadOssRes( file , ResConstants.BizType.AVATAR );
            sysRes.setResMediatype(ResConstants.mediaType.IMG.getValue());
            sysRes.setResBiztype(ResConstants.BizType.AVATAR.getKey());
            sysRes.setResUrl(avatar);
            sysRes.setResOssUrl(resOssUrl);
            if(SpringUtils.getBean(PolicyService.class).isOssCloudPolicy()){
                sysRes.setResUrl(resOssUrl);
            }
            sysRes.setResSeq(1L);
            sysRes.setResCreator(userId);
            sysRes.setResCreateddate(DateUtils.dateTimeNow());
            sysResService.saveAvatarByUserId(sysRes);
            return sysRes;
        }catch (Exception e){
            throw new BizException(ErrorCode.UPLOAD_RESID_FAIL.getCode(),ErrorCode.UPLOAD_RESID_FAIL.getMessage());
        }
    }

    /**
     * 重置用户密码
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(IBaseUser user)
    {
        return sysUserMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     * 
     * @param userLoginname 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userLoginname, String password)
    {
        return sysUserMapper.resetUserPwd(userLoginname, password);
    }

    /**
     * 新增用户角色信息
     * 
     * @param user 用户对象
     */
    public void insertUserRole(IBaseUser user)
    {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     * 
     * @param user 用户对象
     */
    public void insertUserPost(IBaseUser user)
    {

    }

    /**
     * 新增用户角色信息
     * 
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(String userId, String[] roleIds)
    {
        if (StringUtils.isNotEmpty(roleIds))
        {
            // 新增用户与角色管理
            List<IUserRole> list = new ArrayList<IUserRole>(roleIds.length);
            for (String roleId : roleIds)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserRoleId(Uuids.getId());
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

    /**
     * 通过用户ID删除用户
     * 
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(String userId)
    {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        return sysUserMapper.deleteUserById(userId);
    }

    /**
     * 批量删除用户信息
     * 
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(String[] userIds)
    {
        for (String userId : userIds)
        {
            checkUserAllowed(new SysUser(userId));
            checkUserDataScope(userId);
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        return sysUserMapper.deleteUserByIds(userIds);
    }

    /**
     * 导入用户数据
     * 
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<IBaseUser> userList, Boolean isUpdateSupport, String operName)
    {
        if (StringUtils.isNull(userList) || userList.size() == 0)
        {
            throw new BizException(ErrorCode.UNKNOWN_ERROR.getCode(),"导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey(ConfigUtils.ConfigKeyEnum.INIT_PASSWORD.getKey());
        for (IBaseUser user : userList)
        {
            try
            {
                // 验证是否存在这个用户
                IBaseUser u = sysUserMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setUserLoginpwd(SecurityUtils.encryptPassword(password));
                    user.setUserCreator(operName);
                    sysUserMapper.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    checkUserAllowed(u);
                    checkUserDataScope(u.getUserId());
                    user.setUserId(u.getUserId());
                    user.setUserModifier(operName);
                    sysUserMapper.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new BizException(ErrorCode.UNKNOWN_ERROR.getCode(),failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 查询用户绑定
     *
     * @param userId 用户Id
     * @param sysId 系统Id
     * @return 结果
     */
    @Override
    public List<IBaseUserBind> selectPtsUserBind(String userId, String sysId) {
        IBaseUserBind PtsSysuser = new PtsUserBind();
        PtsSysuser.setUserId(userId);
        PtsSysuser.setUserBindSys(sysId);
        return SpringUtils.getBean(PtsUserBindMapper.class).selectPtsSysUserBind(PtsSysuser);
    }

    /**
     * 查询用户绑定
     *
     * @param userId 用户Id
     * @param sysId 系统Id
     * @return 结果
     */
    @Override
    public List<IBaseUserBind> selectPtsUserBind(String userId , String sysId, boolean orgEnabled){
        if( orgEnabled ){
            List<IBaseUserBind> userBinds = new ArrayList<IBaseUserBind>();
            IBaseUser sysUser = this.selectUserById(userId);
            IBaseUserBind userBinder = new PtsUserBind();
            userBinder.setUserBindId(IdUtils.fastUUID());
            userBinder.setUserId(userId );
            userBinder.setUserBindSysUser(userId);
            userBinder.setUserBindSys( sysId );
            userBinds.add( userBinder );
            return userBinds;
        }else{
            return selectPtsUserBind(userId , sysId);
        }
    }

    /**
     * 查询用户绑定数目
     *
     * @param userId 用户Id
     * @param sysId 系统Id
     * @return 结果
     */
    @Override
    public int hasPtsSysUserId(String userId, String sysId,String userbindSysuser) {
        IBaseUserBind PtsSysuser = new PtsUserBind();
        PtsSysuser.setUserId(userId);
        PtsSysuser.setUserBindSys(sysId);
        PtsSysuser.setUserBindSysUser( userbindSysuser );
        return SpringUtils.getBean(PtsUserBindMapper.class).hasPtsSysUserBind(PtsSysuser);
    }


    /**
     * 新增用户绑定
     *
     * @param userId 用户Id
     * @param sysId 系统Id
     * @return 结果
     */
    @Override
    public int insertPtsUserBind(String userId , String sysId , String userbindSysuser){
        IBaseUserBind PtsSysuser = new PtsUserBind();
        PtsSysuser.setUserBindId( PtsSysuser.getId() );
        PtsSysuser.setUserId(userId);
        PtsSysuser.setUserBindSys(sysId);
        PtsSysuser.setUserBindSysUser( userbindSysuser );
        return SpringUtils.getBean(PtsUserBindMapper.class).insertPtsUserBind(PtsSysuser);
    }
    /**
     * 删除用户绑定
     *
     * @param userList 用户Id
     * @return 结果
     */
    @Override
    public int deletePtsUserBind(List<IBaseUserBind> userList){
        return SpringUtils.getBean(PtsUserBindMapper.class).deletePtsUserBind(userList);
    }


}
